require "defines"
require "libs/logger"
require "libs/light"

script.on_event(defines.events.on_tick, function(event)
    update_weather(event.tick)
    local light = get_base_light_level(event.tick) * (1 - global.weather.cloud_cover)

    global.current_light = light
    local time_of_day = light_level_to_time_of_day(light)
    game.daytime = time_of_day
end)


day_phases = {
    {name = "day", duration = 12500, start_light = 1.0, end_light = 1.0}, 
    {name = "dusk", duration = 5000, start_light = 1.0, end_light = 0.14}, 
    {name = "night", duration = 2500, start_light=0.14, end_light = 0.14}, 
    {name = "dawn", duration = 5000, start_light=0.14, end_light = 1.0}
}


total_cycle_length = 0
for i, v in ipairs(day_phases) do
   v.start_tick = total_cycle_length
   total_cycle_length = total_cycle_length + v.duration
end

function get_base_light_level(event_tick)
   local tick = event_tick % total_cycle_length
   for i, phase in ipairs(day_phases) do
      if phase.start_tick <= tick and tick < phase.start_tick + phase.duration then
          local phase_percent = (tick - phase.start_tick) / phase.duration
          return phase.start_light * (1 - phase_percent) + phase.end_light * phase_percent
      end
   end

   Logger.log("Oops! Unable to figure out what time of day it is " .. event_tick)
end

function update_weather(event_tick)
    if global.weather == nil or event_tick >= global.weather.duration_end_tick then
        change_weather(event_tick)
    elseif event_tick <= global.weather.transition_end_tick then
        if global.weather.transition_end_tick == event_tick then
            game.player.print(strings[global.weather.name])
        end

        global.weather.cloud_cover = global.weather.cloud_cover + global.weather.transition_delta
    end
end

weather = {
    clear = {cloud_cover = 0.0, transition_time = 60 * 30, odds = 30, duration = 60 * 60 * 3},
    hazy = {cloud_cover = 0.1, transition_time = 60 * 30, odds = 20, duration = 60 * 60 * 2},
    overcast = {cloud_cover = 0.25, transition_time = 60 * 15, odds = 30, duration = 60 * 60 * 2},
    stormy = {cloud_cover = 0.7, transition_time = 60 * 5, odds = 10, duration = 60 * 60 * 0.5},
    gloomy = {cloud_cover = 0.45, transition_time = 60 * 30, odds = 10, duration = 60 * 60 * 7}
}


strings = {
 clear = "It is now clear",
 hazy = "It is now hazy",
 overcast = "It is now overcast",
 stormy = "It is now stormy",
 gloomy = "Dark ominous clouds fill the sky"
}

function choose_weather()
    local best_roll = 0
    local best = nil

    for name, data in pairs(weather) do
        local roll = math.random(0, data.odds * 100)
        if name == global.weather.name then roll = roll / 2 end
        if roll > best_roll then 
            best = name 
            best_roll = roll
        end
    end
    return best
end


function change_weather(event_tick) 
    if global.weather == nil then global.weather = {cloud_cover = 0} end
    
    local new_weather = choose_weather()
    local weather_data = weather[new_weather]
    
    local old_cloud_cover = global.weather.cloud_cover
    local target_cloud_cover = weather_data.cloud_cover * (9 + 2 * math.random()) / 10
    local transition_duration = math.floor(weather_data.transition_time * (2 + math.random() + math.random()) / 3)
    local duration = math.floor(weather_data.duration * (1 + math.random() + math.random()) / 2)
    
    global.weather.name = new_weather
    global.weather.duration_end_tick = event_tick + duration
    global.weather.transition_end_tick = event_tick + transition_duration
    global.weather.transition_delta = (target_cloud_cover - old_cloud_cover) / transition_duration
end
